package org.modeshape.example.filesystem;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Random;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Repository;
import javax.jcr.Session;

import org.modeshape.common.logging.Logger;
import org.modeshape.common.collection.Problems;
import org.modeshape.jcr.ModeShapeEngine;
import org.modeshape.jcr.RepositoryConfiguration;

public class ModeShapeExample {

    public static void main( String[] argv ) {
        Random rnd = new Random();

        // Create and start the engine ...
        ModeShapeEngine engine = new ModeShapeEngine();
        engine.start();

        // Load the configuration for a repository via the classloader (can also use path to a file)...
        Repository repository = null;
        String repositoryName = null;
        try {
        	
            URL url = ModeShapeExample.class.getClassLoader().getResource("my-repository-config.json");
            if(argv.length>=1 && argv[0].equals("slave")){
            	 url = ModeShapeExample.class.getClassLoader().getResource("my-repository-config-slave.json");
            }
            RepositoryConfiguration config = RepositoryConfiguration.read(url);

            // We could change the name of the repository programmatically ...
            // config = config.withName("Some Other Repository");

            // Verify the configuration for the repository ...
            Problems problems = config.validate();
            if (problems.hasErrors()) {
                System.err.println("Problems starting the engine.");
                System.err.println(problems);
                System.exit(-1);
            }

            // Deploy the repository ...
            repository = engine.deploy(config);
            repositoryName = config.getName();
        } catch (Throwable e) {
            e.printStackTrace();
            System.exit(-1);
            return;
        }

        Session session = null;
        try {
            // Get the repository
            repository = engine.getRepository(repositoryName);
            
            InputStreamReader inp = new InputStreamReader(System.in);
            BufferedReader br = new BufferedReader(inp);
            String s=new String();
            while(!(s=br.readLine()).equals("q")){
            	
            // Create a session ...
            session = repository.login("default");
            // Get the root node ...
            Node root = session.getRootNode();
            assert root != null;

            System.out.println("Found the root node in the \"" + session.getWorkspace().getName() + "\" workspace");
            
           
            Node n = root.addNode("Node" + rnd.nextInt());

            n.setProperty("key", "value");
            n.setProperty("content", session.getValueFactory().createBinary(new ByteArrayInputStream(s.getBytes())));

            session.save();

            System.out.println("Added one node under root");

            
         // Obtain the query manager for the session via the workspace ...
            javax.jcr.query.QueryManager queryManager = session.getWorkspace().getQueryManager();
              
            // Create a query object ...
            String language = javax.jcr.query.Query.JCR_SQL2;
            String expression = "SELECT * FROM [nt:base]	 ";
            javax.jcr.query.Query query = queryManager.createQuery(expression,language);
            javax.jcr.query.QueryResult result = query.execute();
            System.out.println("+from query ---> " +result.getNodes().getSize());
            
            System.out.println("+ Root childs");

            NodeIterator it = root.getNodes();
            System.out.println("+---> " + it.getSize());
//            while (it.hasNext()) {
//                System.out.println("+---> " + it.nextNode().getName());
//            }
            
            session.logout();
            
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (session != null) session.logout();
            System.out.println("Shutting down engine ...");
            try {
                engine.shutdown().get();
                System.out.println("Success!");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

    }
}
